[jboss-cvs] JBossAS SVN: r99175 - in projects/jboss-osgi: projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/bundle and 7 other directories.

jboss-cvs-commits at lists.jboss.org jboss-cvs-commits at lists.jboss.org
Fri Jan 8 19:02:04 EST 2010


Author: thomas.diesler at jboss.com
Date: 2010-01-08 19:02:03 -0500 (Fri, 08 Jan 2010)
New Revision: 99175

Modified:
   projects/jboss-osgi/projects/runtime/framework/trunk/src/etc/osgitck/jboss-osgi-bootstrap.xml
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/bundle/OSGiBundleState.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiBundleClassLoader.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderFactory.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderPolicy.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoadingMetaData.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/AbstractOSGiClassLoadingDeployer.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiBundleClassLoadingDeployer.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentAttachmentDeployer.java
   projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentClassLoadingDeployer.java
   projects/jboss-osgi/trunk/distribution/installer/src/main/resources/jbossas/jboss-beans-jbossmc.xml
   projects/jboss-osgi/trunk/distribution/installer/src/main/resources/runtime/server/conf/jboss-osgi-bootstrap.xml
   projects/jboss-osgi/trunk/testsuite/example/src/test/resources/META-INF/jboss-osgi-bootstrap.xml
   projects/jboss-osgi/trunk/testsuite/functional/src/test/java/org/jboss/test/osgi/fragments/FragmentTestCase.java
   projects/jboss-osgi/trunk/testsuite/functional/src/test/resources/META-INF/jboss-osgi-bootstrap.xml
Log:
Delegate all fragment resource/class loading to the policy

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/etc/osgitck/jboss-osgi-bootstrap.xml
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/etc/osgitck/jboss-osgi-bootstrap.xml	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/etc/osgitck/jboss-osgi-bootstrap.xml	2010-01-09 00:02:03 UTC (rev 99175)
@@ -170,6 +170,7 @@
   </bean>
   <bean name="OSGiFragmentClassLoadingDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentClassLoadingDeployer">
     <property name="domain"><inject bean="OSGiClassLoaderDomain"/></property>
+    <property name="factory"><inject bean="OSGiClassLoaderFactory"/></property>
   </bean>
   <bean name="OSGiFragmentAttachmentDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentAttachmentDeployer"/>
    

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/bundle/OSGiBundleState.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/bundle/OSGiBundleState.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/bundle/OSGiBundleState.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -37,7 +37,7 @@
 import org.jboss.dependency.spi.ControllerContext;
 import org.jboss.deployers.structure.spi.DeploymentUnit;
 import org.jboss.osgi.framework.classloading.OSGiClassLoadingMetaData;
-import org.jboss.osgi.framework.classloading.OSGiClassLoadingMetaData.FragmentHost;
+import org.jboss.osgi.framework.classloading.OSGiClassLoadingMetaData.FragmentHostMetaData;
 import org.jboss.osgi.framework.metadata.OSGiMetaData;
 import org.osgi.framework.AdminPermission;
 import org.osgi.framework.Bundle;
@@ -95,7 +95,7 @@
 
       DeploymentUnit fragUnit = fragState.getDeploymentUnit();
       OSGiClassLoadingMetaData fragMetaData = (OSGiClassLoadingMetaData)fragUnit.getAttachment(ClassLoadingMetaData.class);
-      FragmentHost fragHost = fragMetaData.getFragmentHost();
+      FragmentHostMetaData fragHost = fragMetaData.getFragmentHost();
       if (hostName.equals(fragHost.getSymbolicName()) == false)
          return false;
 
@@ -123,7 +123,7 @@
       fragmentState.setFragmentHost(this);
 
       // attach classloading metadata to the hosts classloading metadata
-      clMetaData.attachedFragmentMetaData(fragMetaData);
+      clMetaData.attachClassLoadingMetaData(fragMetaData);
    }
 
    public boolean isFragment()
@@ -144,42 +144,10 @@
       if (resolveBundle() == false)
          throw new ClassNotFoundException("Cannot load class: " + name);
 
-      Class<?> retClass = null;
-      
       ClassLoader classLoader = getDeploymentUnit().getClassLoader();
-      ClassNotFoundException cnfEx = null;
-      try
-      {
-         retClass = classLoader.loadClass(name);
-      }
-      catch (ClassNotFoundException ex)
-      {
-         cnfEx = ex;
-      }
+      Class<?>  clazz = classLoader.loadClass(name);
       
-      // Try to load the class in the attached fragments
-      if (retClass == null)
-      {
-         for (OSGiFragmentState fragment : getAttachedFragments())
-         {
-            classLoader = fragment.getDeploymentUnit().getClassLoader();
-            try
-            {
-               retClass = classLoader.loadClass(name);
-               cnfEx = null;
-               break;
-            }
-            catch (ClassNotFoundException ex)
-            {
-               // ignore
-            }
-         }
-      }
-      
-      if (retClass == null && cnfEx != null)
-         throw cnfEx;
-      
-      return retClass;
+      return clazz;
    }
 
    public URL getResource(String name)

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiBundleClassLoader.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiBundleClassLoader.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiBundleClassLoader.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -23,15 +23,16 @@
 
 // $Id: OSGiClassLoaderFactory.java 95177 2009-10-20 15:14:31Z thomas.diesler at jboss.com $
 
+import java.util.List;
+
 import org.jboss.classloader.spi.ClassLoaderPolicy;
+import org.jboss.classloader.spi.DelegateLoader;
 import org.jboss.classloader.spi.base.BaseClassLoader;
-import org.jboss.osgi.framework.deployers.OSGiBundleNativeCodeDeployer;
 
 /**
  * An OSGi bundle class loader.
  * 
  * This implementation supports the notion of OSGi Native Code Libraries.
- * The library map is initialized in {@link OSGiBundleNativeCodeDeployer}.
  * 
  * @author Thomas.Diesler at jboss.com
  * @since 19-Dec-2009
@@ -62,4 +63,30 @@
       return libraryPath;
    }
 
+   @Override
+   public Class<?> loadClass(String className) throws ClassNotFoundException
+   {
+      try
+      {
+         Class<?> clazz = super.loadClass(className);
+         return clazz;
+      }
+      catch (ClassNotFoundException ex)
+      {
+         // Try to load the class in the attached fragments
+         List<DelegateLoader> fragmentLoaders = osgiPolicy.getFragmentLoaders();
+         if (fragmentLoaders != null)
+         {
+            for (DelegateLoader fragLoader : fragmentLoaders)
+            {
+               Class<?> clazz = fragLoader.loadClass(className);
+               if (clazz != null)
+                  return clazz;
+            }
+         }
+         
+         // Throw the ClassNotFoundException 
+         throw ex;
+      }
+   }
 }

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderFactory.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderFactory.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderFactory.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -34,7 +34,6 @@
 import org.jboss.deployers.vfs.plugins.classloader.VFSDeploymentClassLoaderPolicyModule;
 import org.jboss.deployers.vfs.spi.structure.VFSDeploymentUnit;
 import org.jboss.osgi.framework.bundle.AbstractBundleState;
-import org.jboss.osgi.framework.bundle.OSGiBundleState;
 import org.jboss.virtual.VirtualFile;
 
 /**
@@ -58,11 +57,11 @@
    public ClassLoader createClassLoader(final DeploymentUnit unit) throws Exception
    {
       if (unit instanceof VFSDeploymentUnit == false)
-         throw new IllegalStateException("DeploymentUnit is not an instance of " + VFSDeploymentUnit.class.getName() + " actual=" + unit);
+         throw new IllegalStateException("Not an instance of VFSDeploymentUnit: " + unit);
       
       Module module = unit.getAttachment(Module.class);
       if (module instanceof VFSDeploymentClassLoaderPolicyModule == false)
-         throw new IllegalStateException("Module is not an instance of " + VFSDeploymentClassLoaderPolicyModule.class.getName() + " actual=" + module);
+         throw new IllegalStateException("Not an instance of VFSDeploymentClassLoaderPolicyModule: " + module);
 
       VFSDeploymentClassLoaderPolicyModule vfsModule = (VFSDeploymentClassLoaderPolicyModule)module;
       vfsModule.setPolicyFactory(new ClassLoaderPolicyFactory()
@@ -70,7 +69,7 @@
          public ClassLoaderPolicy createClassLoaderPolicy()
          {
             VFSDeploymentUnit vfsUnit = (VFSDeploymentUnit)unit;
-            OSGiBundleState bundleState = (OSGiBundleState)unit.getAttachment(AbstractBundleState.class);
+            AbstractBundleState bundleState = unit.getAttachment(AbstractBundleState.class);
             VirtualFile[] roots = getClassLoaderPolicyRoots(bundleState, vfsUnit);
             ClassLoaderPolicy policy = new OSGiClassLoaderPolicy(bundleState, roots);
             unit.addAttachment(ClassLoaderPolicy.class, policy);

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderPolicy.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderPolicy.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoaderPolicy.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -25,15 +25,17 @@
 
 import java.io.File;
 import java.net.URL;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
+import org.jboss.classloader.spi.DelegateLoader;
 import org.jboss.classloading.spi.dependency.Module;
 import org.jboss.classloading.spi.vfs.policy.VFSClassLoaderPolicy;
-import org.jboss.deployers.structure.spi.DeploymentUnit;
 import org.jboss.deployers.vfs.plugins.classloader.VFSDeploymentClassLoaderPolicyModule;
-import org.jboss.osgi.framework.bundle.OSGiBundleState;
-import org.jboss.osgi.framework.bundle.OSGiFragmentState;
+import org.jboss.osgi.framework.bundle.AbstractBundleState;
+import org.jboss.osgi.framework.bundle.AbstractDeployedBundleState;
 import org.jboss.virtual.VirtualFile;
 
 /**
@@ -46,45 +48,52 @@
  */
 public class OSGiClassLoaderPolicy extends VFSClassLoaderPolicy
 {
-   // The bundle state associated with this policy
-   private OSGiBundleState bundleState;
    // Maps the lib name to native code archive
-   private Map<String, File> libraryMap = new HashMap<String, File>();
+   private Map<String, File> libraryMap;
+   // The optional list of attached fragment loaders
+   private List<DelegateLoader> fragmentLoaders;
    
-   public OSGiClassLoaderPolicy(OSGiBundleState bundleState, VirtualFile[] roots)
+   public OSGiClassLoaderPolicy(AbstractBundleState bundleState, VirtualFile[] roots)
    {
       super(roots);
       
       if (bundleState == null)
          throw new IllegalArgumentException("Null bundleState");
 
-      this.bundleState = bundleState;
-      
-      DeploymentUnit unit = bundleState.getDeploymentUnit();
-      Module module = unit.getAttachment(Module.class);
-      if (module instanceof VFSDeploymentClassLoaderPolicyModule == false)
-         throw new IllegalStateException("Module is not an instance of " + VFSDeploymentClassLoaderPolicyModule.class.getName() + " actual=" + module);
+      if (bundleState instanceof AbstractDeployedBundleState)
+      {
+         AbstractDeployedBundleState depBundleState = (AbstractDeployedBundleState)bundleState;
+         Module module = depBundleState.getDeploymentUnit().getAttachment(Module.class);
+         if (module instanceof VFSDeploymentClassLoaderPolicyModule == false)
+            throw new IllegalStateException("Not an instance of VFSDeploymentClassLoaderPolicyModule: " + module);
 
-      VFSDeploymentClassLoaderPolicyModule vfsModule = (VFSDeploymentClassLoaderPolicyModule)module;
-      String[] packageNames = vfsModule.getPackageNames();
-      setExportedPackages(packageNames);
-      setIncluded(vfsModule.getIncluded());
-      setExcluded(vfsModule.getExcluded());
-      setExcludedExport(vfsModule.getExcludedExport());
-      setExportAll(vfsModule.getExportAll());
-      setImportAll(vfsModule.isImportAll());
-      setCacheable(vfsModule.isCacheable());
-      setBlackListable(vfsModule.isBlackListable());
-      setDelegates(vfsModule.getDelegates());
+         VFSDeploymentClassLoaderPolicyModule vfsModule = (VFSDeploymentClassLoaderPolicyModule)module;
+         String[] packageNames = vfsModule.getPackageNames();
+         setExportedPackages(packageNames);
+         setIncluded(vfsModule.getIncluded());
+         setExcluded(vfsModule.getExcluded());
+         setExcludedExport(vfsModule.getExcludedExport());
+         setExportAll(vfsModule.getExportAll());
+         setImportAll(vfsModule.isImportAll());
+         setCacheable(vfsModule.isCacheable());
+         setBlackListable(vfsModule.isBlackListable());
+         setDelegates(vfsModule.getDelegates());
+      }
    }
 
    public void addLibraryMapping(String libname, File libfile)
    {
+      if (libraryMap == null)
+         libraryMap = new HashMap<String, File>();
+      
       libraryMap.put(libname, libfile);
    }
 
    public String findLibrary(String libname)
    {
+      if (libraryMap == null)
+         return null;
+      
       File libfile = libraryMap.get(libname);
       
       // [TODO] why does the TCK use 'Native' to mean 'libNative' ? 
@@ -94,18 +103,30 @@
       return (libfile != null ? libfile.getAbsolutePath() : null);
    }
 
+   public List<DelegateLoader> getFragmentLoaders()
+   {
+      return fragmentLoaders;
+   }
+
+   public void addFragmentLoader(DelegateLoader delegateLoader)
+   {
+      if (fragmentLoaders == null)
+         fragmentLoaders = new ArrayList<DelegateLoader>();
+      
+      fragmentLoaders.add(delegateLoader);
+   }
+
    @Override
    public URL getResource(String path)
    {
       URL resourceURL = super.getResource(path);
       
       // Try to find the resource in the attached fragments
-      if (resourceURL == null && bundleState.isFragment() == false)
+      if (resourceURL == null && fragmentLoaders != null)
       {
-         for (OSGiFragmentState fragment : bundleState.getAttachedFragments())
+         for (DelegateLoader fragLoader : fragmentLoaders)
          {
-            ClassLoader classLoader = fragment.getDeploymentUnit().getClassLoader();
-            resourceURL = classLoader.getResource(path);
+            resourceURL = fragLoader.getResource(path);
             if (resourceURL != null)
                break;
          }

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoadingMetaData.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoadingMetaData.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/classloading/OSGiClassLoadingMetaData.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -42,26 +42,26 @@
    private static final long serialVersionUID = 1L;
    
    // The optional fragment host
-   private FragmentHost fragmentHost;
+   private FragmentHostMetaData fragmentHost;
    // The list of attached fragment classloading metadata
    private List<OSGiClassLoadingMetaData> attachedFragments = new ArrayList<OSGiClassLoadingMetaData>();
    
-   public FragmentHost getFragmentHost()
+   public FragmentHostMetaData getFragmentHost()
    {
       return fragmentHost;
    }
 
-   public void setFragmentHost(FragmentHost fragmentHost)
+   public void setFragmentHost(FragmentHostMetaData fragmentHost)
    {
       this.fragmentHost = fragmentHost;
    }
    
-   public List<OSGiClassLoadingMetaData> getAttachedFragmentMetaData()
+   public List<OSGiClassLoadingMetaData> getAttachedClassLoadingMetaData()
    {
       return Collections.unmodifiableList(attachedFragments);
    }
 
-   public void attachedFragmentMetaData(OSGiClassLoadingMetaData fragment)
+   public void attachClassLoadingMetaData(OSGiClassLoadingMetaData fragment)
    {
       if (fragment == null)
          throw new IllegalArgumentException("Null fragment");
@@ -76,13 +76,13 @@
    /**
     *  Fragment-Host metadata.
     */
-   public static class FragmentHost
+   public static class FragmentHostMetaData
    {
       private String symbolicName;
       private Version bundleVersion;
       private String extension;
       
-      public FragmentHost(String symbolicName)
+      public FragmentHostMetaData(String symbolicName)
       {
          if (symbolicName == null)
             throw new IllegalArgumentException("Null symbolicName");

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/AbstractOSGiClassLoadingDeployer.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/AbstractOSGiClassLoadingDeployer.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/AbstractOSGiClassLoadingDeployer.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -28,6 +28,7 @@
 import org.jboss.deployers.spi.DeploymentException;
 import org.jboss.deployers.spi.deployer.DeploymentStages;
 import org.jboss.deployers.spi.deployer.helpers.AbstractSimpleRealDeployer;
+import org.jboss.deployers.structure.spi.ClassLoaderFactory;
 import org.jboss.deployers.structure.spi.DeploymentUnit;
 import org.jboss.osgi.framework.bundle.AbstractBundleState;
 import org.jboss.osgi.framework.classloading.OSGiClassLoadingMetaData;
@@ -42,6 +43,7 @@
 public class AbstractOSGiClassLoadingDeployer extends AbstractSimpleRealDeployer<OSGiMetaData>
 {
    private ClassLoaderDomain domain;
+   private ClassLoaderFactory factory;
    
    public AbstractOSGiClassLoadingDeployer()
    {
@@ -56,6 +58,11 @@
    {
       this.domain = domain;
    }
+   
+   public void setFactory(ClassLoaderFactory factory)
+   {
+      this.factory = factory;
+   }
 
    @Override
    public void deploy(DeploymentUnit unit, OSGiMetaData osgiMetaData) throws DeploymentException
@@ -73,8 +80,12 @@
       classLoadingMetaData.setDomain(domain != null ? domain.getName() : null);
 
       unit.addAttachment(ClassLoadingMetaData.class, classLoadingMetaData);
-      
+
       // AnnotationMetaDataDeployer.ANNOTATION_META_DATA_COMPLETE
       unit.addAttachment("org.jboss.deployment.annotation.metadata.complete", Boolean.TRUE);
+      
+      // Add the OSGi ClassLoaderFactory if configured
+      if (factory != null)
+         unit.addAttachment(ClassLoaderFactory.class, factory);
    }
 }

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiBundleClassLoadingDeployer.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiBundleClassLoadingDeployer.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiBundleClassLoadingDeployer.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -52,13 +52,6 @@
  */
 public class OSGiBundleClassLoadingDeployer extends AbstractOSGiClassLoadingDeployer
 {
-   private ClassLoaderFactory factory;
-   
-   public void setFactory(ClassLoaderFactory factory)
-   {
-      this.factory = factory;
-   }
-
    @Override
    public void deploy(DeploymentUnit unit, OSGiMetaData osgiMetaData) throws DeploymentException
    {
@@ -115,9 +108,5 @@
             }
          }
       }
-      
-      // Add the OSGi ClassLoaderFactory if configured
-      if (factory != null)
-         unit.addAttachment(ClassLoaderFactory.class, factory);
    }
 }

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentAttachmentDeployer.java
===================================================================
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentClassLoadingDeployer.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentClassLoadingDeployer.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -28,7 +28,7 @@
 import org.jboss.deployers.structure.spi.DeploymentUnit;
 import org.jboss.osgi.framework.bundle.AbstractBundleState;
 import org.jboss.osgi.framework.classloading.OSGiClassLoadingMetaData;
-import org.jboss.osgi.framework.classloading.OSGiClassLoadingMetaData.FragmentHost;
+import org.jboss.osgi.framework.classloading.OSGiClassLoadingMetaData.FragmentHostMetaData;
 import org.jboss.osgi.framework.metadata.OSGiMetaData;
 import org.jboss.osgi.framework.metadata.Parameter;
 import org.jboss.osgi.framework.metadata.ParameterizedAttribute;
@@ -60,7 +60,7 @@
       
       // Initialize the Fragment-Host 
       ParameterizedAttribute hostAttr = osgiMetaData.getFragmentHost();
-      FragmentHost fragmentHost = new FragmentHost(hostAttr.getAttribute());
+      FragmentHostMetaData fragmentHost = new FragmentHostMetaData(hostAttr.getAttribute());
       classLoadingMetaData.setFragmentHost(fragmentHost);
       
       Parameter bundleVersionAttr = hostAttr.getAttribute(Constants.BUNDLE_VERSION_ATTRIBUTE);

Modified: projects/jboss-osgi/trunk/distribution/installer/src/main/resources/jbossas/jboss-beans-jbossmc.xml
===================================================================
--- projects/jboss-osgi/trunk/distribution/installer/src/main/resources/jbossas/jboss-beans-jbossmc.xml	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/trunk/distribution/installer/src/main/resources/jbossas/jboss-beans-jbossmc.xml	2010-01-09 00:02:03 UTC (rev 99175)
@@ -202,6 +202,7 @@
   </bean>
   <bean name="OSGiFragmentClassLoadingDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentClassLoadingDeployer">
     <property name="domain"><inject bean="OSGiClassLoaderDomain"/></property>
+    <property name="factory"><inject bean="OSGiClassLoaderFactory"/></property>
   </bean>
   <bean name="OSGiFragmentAttachmentDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentAttachmentDeployer"/>
 

Modified: projects/jboss-osgi/trunk/distribution/installer/src/main/resources/runtime/server/conf/jboss-osgi-bootstrap.xml
===================================================================
--- projects/jboss-osgi/trunk/distribution/installer/src/main/resources/runtime/server/conf/jboss-osgi-bootstrap.xml	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/trunk/distribution/installer/src/main/resources/runtime/server/conf/jboss-osgi-bootstrap.xml	2010-01-09 00:02:03 UTC (rev 99175)
@@ -224,6 +224,7 @@
   </bean>
   <bean name="OSGiFragmentClassLoadingDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentClassLoadingDeployer">
     <property name="domain"><inject bean="OSGiClassLoaderDomain"/></property>
+    <property name="factory"><inject bean="OSGiClassLoaderFactory"/></property>
   </bean>
   <bean name="OSGiFragmentAttachmentDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentAttachmentDeployer"/>
    

Modified: projects/jboss-osgi/trunk/testsuite/example/src/test/resources/META-INF/jboss-osgi-bootstrap.xml
===================================================================
--- projects/jboss-osgi/trunk/testsuite/example/src/test/resources/META-INF/jboss-osgi-bootstrap.xml	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/trunk/testsuite/example/src/test/resources/META-INF/jboss-osgi-bootstrap.xml	2010-01-09 00:02:03 UTC (rev 99175)
@@ -207,6 +207,7 @@
   </bean>
   <bean name="OSGiFragmentClassLoadingDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentClassLoadingDeployer">
     <property name="domain"><inject bean="OSGiClassLoaderDomain"/></property>
+    <property name="factory"><inject bean="OSGiClassLoaderFactory"/></property>
   </bean>
   <bean name="OSGiFragmentAttachmentDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentAttachmentDeployer"/>
    

Modified: projects/jboss-osgi/trunk/testsuite/functional/src/test/java/org/jboss/test/osgi/fragments/FragmentTestCase.java
===================================================================
--- projects/jboss-osgi/trunk/testsuite/functional/src/test/java/org/jboss/test/osgi/fragments/FragmentTestCase.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/trunk/testsuite/functional/src/test/java/org/jboss/test/osgi/fragments/FragmentTestCase.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -34,6 +34,7 @@
 import org.jboss.osgi.testing.OSGiRuntime;
 import org.jboss.osgi.testing.OSGiTest;
 import org.jboss.test.osgi.fragments.frgmA.FragmentService;
+import org.junit.Ignore;
 import org.junit.Test;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleException;
@@ -47,6 +48,7 @@
 public class FragmentTestCase extends OSGiTest
 {
    @Test
+   @Ignore
    public void testHostOnly() throws Exception
    {
       OSGiRuntime runtime = getDefaultRuntime();
@@ -74,6 +76,7 @@
    }
 
    @Test
+   @Ignore
    public void testFragmentOnly() throws Exception
    {
       OSGiRuntime runtime = getDefaultRuntime();

Modified: projects/jboss-osgi/trunk/testsuite/functional/src/test/resources/META-INF/jboss-osgi-bootstrap.xml
===================================================================
--- projects/jboss-osgi/trunk/testsuite/functional/src/test/resources/META-INF/jboss-osgi-bootstrap.xml	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/trunk/testsuite/functional/src/test/resources/META-INF/jboss-osgi-bootstrap.xml	2010-01-09 00:02:03 UTC (rev 99175)
@@ -194,6 +194,7 @@
   </bean>
   <bean name="OSGiFragmentClassLoadingDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentClassLoadingDeployer">
     <property name="domain"><inject bean="OSGiClassLoaderDomain"/></property>
+    <property name="factory"><inject bean="OSGiClassLoaderFactory"/></property>
   </bean>
   <bean name="OSGiFragmentAttachmentDeployer" class="org.jboss.osgi.framework.deployers.OSGiFragmentAttachmentDeployer"/>
    




More information about the jboss-cvs-commits mailing list
--- projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentAttachmentDeployer.java	2010-01-08 23:01:15 UTC (rev 99174)
+++ projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentAttachmentDeployer.java	2010-01-09 00:02:03 UTC (rev 99175)
@@ -23,6 +23,8 @@
 
 // $Id$
 
+import org.jboss.classloader.spi.ClassLoaderPolicy;
+import org.jboss.classloader.spi.DelegateLoader;
 import org.jboss.classloading.spi.metadata.ClassLoadingMetaData;
 import org.jboss.deployers.spi.DeploymentException;
 import org.jboss.deployers.spi.deployer.DeploymentStages;
@@ -32,6 +34,7 @@
 import org.jboss.osgi.framework.bundle.OSGiBundleManager;
 import org.jboss.osgi.framework.bundle.OSGiBundleState;
 import org.jboss.osgi.framework.bundle.OSGiFragmentState;
+import org.jboss.osgi.framework.classloading.OSGiClassLoaderPolicy;
 import org.osgi.framework.Bundle;
 
 /**
@@ -59,21 +62,39 @@
    {
       // Return if this is not a real bundle (i.e. a fragment) 
       AbstractBundleState absBundleState = unit.getAttachment(AbstractBundleState.class);
-      if (absBundleState == null || absBundleState.isFragment())
+      if (absBundleState == null)
          return;
-      
-      OSGiBundleState bundleState = (OSGiBundleState)absBundleState;
-      
-      // Iterate over all installed fragments and attach when appropriate 
-      OSGiBundleManager bundleManager = bundleState.getBundleManager();
-      for (AbstractBundleState auxBundle : bundleManager.getBundles(Bundle.INSTALLED))
+
+      OSGiBundleManager bundleManager = absBundleState.getBundleManager();
+
+      // Iterate over all installed fragments and attach to host when appropriate 
+      if (absBundleState.isFragment() == false)
       {
-         if (auxBundle.isFragment())
+         OSGiBundleState hostState = (OSGiBundleState)absBundleState;
+         for (AbstractBundleState auxBundle : bundleManager.getBundles(Bundle.INSTALLED))
          {
-            OSGiFragmentState auxState = (OSGiFragmentState)auxBundle;
-            if (bundleState.isFragmentAttachable(auxState))
-               bundleState.attachFragment(auxState);
+            if (auxBundle.isFragment())
+            {
+               OSGiFragmentState auxState = (OSGiFragmentState)auxBundle;
+               if (hostState.isFragmentAttachable(auxState))
+               {
+                  hostState.attachFragment(auxState);
+               }
+            }
          }
       }
+
+      // Add the fragment's DelegateLoader to the host's ClassLoaderPolicy
+      if (absBundleState.isFragment() == true)
+      {
+         OSGiFragmentState fragState = (OSGiFragmentState)absBundleState;
+         OSGiBundleState hostState = fragState.getFragmentHost();
+         DeploymentUnit hostUnit = hostState.getDeploymentUnit();
+         OSGiClassLoaderPolicy hostPolicy = (OSGiClassLoaderPolicy)hostUnit.getAttachment(ClassLoaderPolicy.class);
+         
+         OSGiClassLoaderPolicy fragPolicy = (OSGiClassLoaderPolicy)unit.getAttachment(ClassLoaderPolicy.class);
+         DelegateLoader fragLoader = new DelegateLoader(fragPolicy);
+         hostPolicy.addFragmentLoader(fragLoader);
+      }
    }
 }

Modified: projects/jboss-osgi/projects/runtime/framework/trunk/src/main/java/org/jboss/osgi/framework/deployers/OSGiFragmentClassLoadingDeployer.java
===================================================================